from unittest import TestCase

from piperider_cli.cli_utils.compare_with_recipe import parse_compare_ref
from piperider_cli.recipes import update_select_with_cli_option
from piperider_cli.recipes.default_recipe_generator import _prepare_dbt_cmds, _prepare_piperider_cmd


class TestRecipe(TestCase):

    def test_update_select_with_cli_option(self):
        options = dict(skip_datasource_connection=False, select=None)
        select = update_select_with_cli_option(options)
        self.assertEqual(('state:modified+',), select)

        options = dict(skip_datasource_connection=False, select=())
        select = update_select_with_cli_option(options)
        self.assertEqual(('state:modified+',), select)

        options = dict(skip_datasource_connection=False, select=('orders',))
        select = update_select_with_cli_option(options)
        self.assertEqual(('orders',), select)

        options = dict(skip_datasource_connection=True, select=None)
        select = update_select_with_cli_option(options)
        self.assertEqual((), select)

        options = dict(skip_datasource_connection=True, select=('orders',))
        select = update_select_with_cli_option(options)
        self.assertEqual((), select)

    def test_prepare_piperider_cmd(self):
        options = dict(base_ref=None, target_ref=None, skip_datasource_connection=False)

        # base
        dbt_field = _prepare_dbt_cmds(options)
        piperider_field = _prepare_piperider_cmd(options)

        self.assertEqual(['dbt deps', 'dbt build'], dbt_field.commands)
        self.assertEqual(['piperider run'], piperider_field.commands)

        # target
        dbt_field = _prepare_dbt_cmds(options, target=True)
        piperider_field = _prepare_piperider_cmd(options)

        self.assertEqual(['dbt deps', 'dbt build --select state:modified+ --state <DBT_STATE_PATH>'],
                         dbt_field.commands)
        self.assertEqual(['piperider run'], piperider_field.commands)

        options['select'] = ('orders',)
        # base
        dbt_field = _prepare_dbt_cmds(options)
        piperider_field = _prepare_piperider_cmd(options)

        self.assertEqual(['dbt deps', 'dbt build'], dbt_field.commands)
        self.assertEqual(['piperider run'], piperider_field.commands)

        # target
        dbt_field = _prepare_dbt_cmds(options, target=True)
        piperider_field = _prepare_piperider_cmd(options)

        self.assertEqual(['dbt deps', 'dbt build --select state:modified+ --state <DBT_STATE_PATH>'],
                         dbt_field.commands)
        self.assertEqual(['piperider run'], piperider_field.commands)

        options['skip_datasource_connection'] = True
        # base
        dbt_field = _prepare_dbt_cmds(options)
        piperider_field = _prepare_piperider_cmd(options)

        self.assertEqual(['dbt deps', 'dbt parse'], dbt_field.commands)
        self.assertEqual(['piperider run --skip-datasource'], piperider_field.commands)

        # target
        dbt_field = _prepare_dbt_cmds(options, target=True)
        piperider_field = _prepare_piperider_cmd(options)

        self.assertEqual(['dbt deps', 'dbt parse'],
                         dbt_field.commands)
        self.assertEqual(['piperider run --skip-datasource'], piperider_field.commands)

    def test_parse_compare_ref(self):
        refs = None
        base_ref, target_ref = parse_compare_ref(refs)
        self.assertEqual(None, base_ref)
        self.assertEqual(None, target_ref)

        refs = 'master..develop'
        base_ref, target_ref = parse_compare_ref(refs)
        self.assertEqual(None, base_ref)
        self.assertEqual(None, target_ref)

        refs = 'master...'
        err, _ = parse_compare_ref(refs)
        self.assertEqual(None, base_ref)
        self.assertEqual(None, target_ref)

        refs = 'master...develop'
        base_ref, target_ref = parse_compare_ref(refs)
        self.assertEqual('master', base_ref)
        self.assertEqual('develop', target_ref)

        refs = 'master'
        base_ref, target_ref = parse_compare_ref(refs)
        self.assertEqual('master', base_ref)
        self.assertEqual(None, target_ref)



import io
from typing import Union

from rich.console import Console

from piperider_cli.cli_utils.cloud import CloudConnectorHelper


class DbtUtil:

    @staticmethod
    def get_dbt_project_path(dbt_project_dir: str = None, no_auto_search: bool = False,
                             recursive: bool = True) -> str:
        import piperider_cli.dbtutil as u
        return u.get_dbt_project_path(dbt_project_dir=dbt_project_dir, no_auto_search=no_auto_search,
                                      recursive=recursive)

    @staticmethod
    def read_dbt_resources(source: Union[str, io.TextIOWrapper, list]):
        import piperider_cli.dbtutil as u
        return u.read_dbt_resources(source=source)

    @staticmethod
    def load_dbt_project(path: str):
        import piperider_cli.dbtutil as u
        return u.load_dbt_project(path)

    @staticmethod
    def load_dbt_profile(path: str):
        import piperider_cli.dbtutil as u
        return u.load_dbt_profile(path)

    @staticmethod
    def load_credential_from_dbt_profile(dbt_profile, profile_name, target_name):
        import piperider_cli.dbtutil as u
        return u.load_credential_from_dbt_profile(dbt_profile=dbt_profile, profile_name=profile_name,
                                                  target_name=target_name)


def verify_upload_related_options(**kwargs):
    console = Console()
    upload = kwargs.get('upload', False)
    upload_with_share = kwargs.get('share', False)
    enable_quick_look_share = kwargs.get('enable_quick_look_share', False)
    enable_auto_upload = CloudConnectorHelper.is_auto_upload()
    is_cloud_login = CloudConnectorHelper.is_login()

    if is_cloud_login is True:
        if enable_auto_upload is True:
            console.print('[[bold green]Enable Auto Upload[/bold green]]')
            upload = True
        if upload_with_share is True:
            upload = True
        return upload, upload_with_share
    else:
        if upload is False and upload_with_share is True and enable_quick_look_share is True:
            # Upload to Cloud Quick Look without login
            console.print(
                '[[bold green]Enable Quick Look Share[/bold green]] '
                'Reports will be uploaded as a temporary quick look.')
            return True, True
        if upload is True or upload_with_share is True:
            console.print(
                '[[bold yellow]Warning[/bold yellow]] '
                'The report won\'t be uploaded due to not logged in.')

        return False, False



import base64
import json
from typing import Optional

import google.auth
import sqlalchemy
from google.api_core import client_info
from google.auth import impersonated_credentials
from google.cloud import bigquery
from google.oauth2 import service_account

USER_AGENT_TEMPLATE = "sqlalchemy/{}"
SCOPES = (
    "https://www.googleapis.com/auth/bigquery",
    "https://www.googleapis.com/auth/cloud-platform",
    "https://www.googleapis.com/auth/drive",
)

target_service_account_email: Optional[str] = None


def google_client_info():
    user_agent = USER_AGENT_TEMPLATE.format(sqlalchemy.__version__)
    return client_info.ClientInfo(user_agent=user_agent)


def create_impersonated_bigquery_client(
        credentials_info=None,
        credentials_path=None,
        credentials_base64=None,
        default_query_job_config=None,
        location=None,
        project_id=None,
):
    default_project = None

    if credentials_base64:
        credentials_info = json.loads(base64.b64decode(credentials_base64))

    if credentials_path:
        credentials = service_account.Credentials.from_service_account_file(
            credentials_path
        )
        credentials = credentials.with_scopes(SCOPES)
        default_project = credentials.project_id
    elif credentials_info:
        credentials = service_account.Credentials.from_service_account_info(
            credentials_info
        )
        credentials = credentials.with_scopes(SCOPES)
        default_project = credentials.project_id
    else:
        credentials, default_project = google.auth.default(scopes=SCOPES)

    if project_id is None:
        project_id = default_project

    if target_service_account_email:
        impersonated_creds = impersonated_credentials.Credentials(
            source_credentials=credentials,
            target_principal=target_service_account_email,
            target_scopes=SCOPES,
            lifetime=3600  # Duration for which the token is valid (in seconds)
        )
        return bigquery.Client(
            client_info=google_client_info(),
            project=project_id,
            credentials=impersonated_creds,
            location=location,
            default_query_job_config=default_query_job_config,
        )

    return bigquery.Client(
        client_info=google_client_info(),
        project=project_id,
        credentials=credentials,
        location=location,
        default_query_job_config=default_query_job_config,
    )



import os.path

from rich.console import Console

from piperider_cli.configuration import Configuration
from piperider_cli.error import RecipeConfigException
from piperider_cli.event import CompareEventPayload
from piperider_cli.recipes import select_recipe_file, RecipeConfiguration, execute_recipe_configuration
from piperider_cli.recipes.default_recipe_generator import generate_default_recipe, show_recipe_content

console = Console()


class RecipeExecutor:
    @staticmethod
    def exec(recipe_name: str, select: tuple = None, modified: bool = False, base_ref: str = None,
             target_ref: str = None, skip_datasource_connection: bool = False, debug=False, event_payload=CompareEventPayload()) -> RecipeConfiguration:
        config = Configuration.instance()
        recipe_path = select_recipe_file(recipe_name)

        event_payload.step = "prepare recipe"
        ds = config.get_datasource()
        if ds is not None:
            event_payload.datasource_type = ds.type_name
        if skip_datasource_connection:
            event_payload.skip_datasource = True

        if recipe_name and (select or modified or base_ref or skip_datasource_connection):
            console.print(
                "[[bold yellow]Warning[/bold yellow]] "
                "The recipe will be ignored when '--select', '--modified', '--base-branch', "
                "or '--skip-datasource' is provided."
            )
        if not skip_datasource_connection and select:
            console.print(
                f"[[bold green]Select[/bold green]] Manually select the dbt nodes to run by '{','.join(select)}'")
        if recipe_path is None or select or modified or base_ref or skip_datasource_connection:
            dbt_project_path = None
            if config.dataSources and config.dataSources[0].args.get('dbt'):
                dbt_project_path = os.path.relpath(config.dataSources[0].args.get('dbt', {}).get('projectDir'))
            # generate a default recipe
            console.rule("Recipe executor: generate recipe")
            options = dict(
                base_ref=base_ref,
                target_ref=target_ref,
                skip_datasource_connection=skip_datasource_connection
            )
            if select:
                options['select'] = select
            recipe = generate_default_recipe(overwrite_existing=False,
                                             dbt_project_path=dbt_project_path,
                                             options=options)
            if recipe is None:
                raise RecipeConfigException(
                    message='Default recipe generation failed.',
                    hint='Please provide a recipe file or feedback an issue to us'
                )
            show_recipe_content(recipe)
        else:
            recipe = RecipeConfiguration.load(recipe_path)

        options = dict(skip_datasource_connection=skip_datasource_connection, select=select)
        execute_recipe_configuration(recipe, options, debug=debug, event_payload=event_payload)

        return recipe
